"""
CPU向量场计算模块 - 提供基于CPU的向量场计算功能
"""
import numpy as np
from typing import Tuple, Union, List, Optional, Any
from ..core.config import config_manager
from ..core.events import Event, EventType, event_bus

class CPUVectorFieldCalculator:
    """CPU向量场计算器"""
    def __init__(self):
        self._event_bus = event_bus
        self._config_manager = config_manager

    def sum_adjacent_vectors(self, grid: np.ndarray, x: int, y: int, include_self: bool = None,
                           self_weight: float = 1.0, neighbor_weight: float = 0.1, 
                           enable_average: bool = False) -> Tuple[float, float]:
        """
        读取目标 (x,y) 的上下左右四个相邻格子的向量并相加（越界安全）。
        返回 (sum_x, sum_y) 的 tuple。
        """
        if grid is None:
            return (0.0, 0.0)

        if not isinstance(grid, np.ndarray):
            raise TypeError("grid 必须是 numpy.ndarray 类型")

        h, w = grid.shape[:2]
        sum_x = 0.0
        sum_y = 0.0
        count = 0

        if include_self and 0 <= x < w and 0 <= y < h:
            vx, vy = float(grid[y, x, 0]), float(grid[y, x, 1])
            sum_x += vx * self_weight
            sum_y += vy * self_weight
            count += 1

        neighbors = ((0, -1), (0, 1), (-1, 0), (1, 0))
        for dx, dy in neighbors:
            nx = x + dx
            ny = y + dy
            if 0 <= nx < w and 0 <= ny < h:
                vx, vy = float(grid[ny, nx, 0]), float(grid[ny, nx, 1])
                sum_x += vx * neighbor_weight
                sum_y += vy * neighbor_weight
                count += 1

        # 如果启用平均值功能，则除以有效向量数量
        if enable_average and count > 0:
            sum_x /= count
            sum_y /= count

        return (sum_x, sum_y)

    def update_grid_with_adjacent_sum(self, grid: np.ndarray, include_self: bool = None) -> np.ndarray:
        """
        使用NumPy的向量化操作高效计算相邻向量之和，替换原有的双重循环实现。
        返回修改后的 grid。
        """
        if grid is None or not isinstance(grid, np.ndarray):
            return grid

        h, w = grid.shape[:2]

        # 获取配置参数
        neighbor_weight = self._config_manager.get("vector_neighbor_weight", 0.1)
        self_weight = self._config_manager.get("vector_self_weight", 1.0)
        enable_average = self._config_manager.get("enable_vector_average", False)
        enable_normalization = self._config_manager.get("enable_vector_normalization", True)

        # 如果未指定include_self，则使用配置管理器中的默认值
        if include_self is None:
            include_self = self._config_manager.get("include_self", False)

        # 预分配结果数组以提高性能
        result = np.zeros_like(grid)

        # 使用向量化操作计算邻居向量之和
        # 创建填充数组来处理边界条件
        padded_grid = np.pad(grid, ((1, 1), (1, 1), (0, 0)), mode='constant')

        # 计算四个方向的邻居贡献
        # 上邻居 (y+1, x)
        up_neighbors = padded_grid[2:, 1:-1] * neighbor_weight
        # 下邻居 (y, x)
        down_neighbors = padded_grid[:-2, 1:-1] * neighbor_weight
        # 左邻居 (y, x+1)
        left_neighbors = padded_grid[1:-1, 2:] * neighbor_weight
        # 右邻居 (y, x)
        right_neighbors = padded_grid[1:-1, :-2] * neighbor_weight

        # 求和邻居贡献
        result = up_neighbors + down_neighbors + left_neighbors + right_neighbors

        # 如果包含自身，添加自身贡献
        if include_self:
            result += grid * self_weight

        # 如果需要平均值，计算有效邻居数量并归一化
        if enable_average:
            # 创建有效邻居计数矩阵
            neighbor_count = np.ones((h, w), dtype=np.float32) * 4  # 默认4个邻居

            # 边界处理
            neighbor_count[0, :] -= 1   # 上边界
            neighbor_count[-1, :] -= 1  # 下边界
            neighbor_count[:, 0] -= 1   # 左边界
            neighbor_count[:, -1] -= 1  # 右边界

            # 如果包含自身，邻居数加1
            if include_self:
                neighbor_count += 1

            # 归一化
            result = result / neighbor_count[:, :, np.newaxis]

        # 如果需要权重归一化
        elif enable_normalization:
            # 计算每个点的有效权重总和
            weight_sum = np.ones((h, w), dtype=np.float32) * 4 * neighbor_weight  # 默认4个邻居

            # 边界处理
            weight_sum[0, :] -= neighbor_weight   # 上边界
            weight_sum[-1, :] -= neighbor_weight  # 下边界
            weight_sum[:, 0] -= neighbor_weight   # 左边界
            weight_sum[:, -1] -= neighbor_weight  # 右边界

            # 如果包含自身，添加自身权重
            if include_self:
                weight_sum += self_weight

            # 确保权重总和大于0，避免除以0
            weight_sum = np.maximum(weight_sum, 0.1)

            # 归一化
            result = result / weight_sum[:, :, np.newaxis]

        # 将结果复制回原网格
        grid[:] = result
        return grid

    def create_vector_grid(self, width: int = 640, height: int = 480, default: Tuple[float, float] = (0, 0)) -> np.ndarray:
        """创建一个 height x width 的二维向量网格"""
        grid = np.zeros((height, width, 2), dtype=np.float32)
        if default != (0, 0):
            grid[:, :, 0] = default[0]
            grid[:, :, 1] = default[1]
        return grid

    def create_radial_pattern(self, grid: np.ndarray, center: Tuple[float, float] = None,
                            radius: float = None, magnitude: float = 1.0) -> np.ndarray:
        """在网格上创建径向向量模式"""
        if grid is None or not isinstance(grid, np.ndarray):
            raise TypeError("grid 必须是 numpy.ndarray 类型")

        h, w = grid.shape[:2]

        # 如果未指定中心，则使用网格中心
        if center is None:
            center = (w // 2, h // 2)

        # 如果未指定半径，则使用网格尺寸的1/4
        if radius is None:
            radius = min(w, h) // 4

        cx, cy = center

        # 创建坐标网格
        y_coords, x_coords = np.mgrid[0:h, 0:w]

        # 计算每个点到中心的距离和方向
        dx = x_coords - cx
        dy = y_coords - cy
        dist = np.sqrt(dx**2 + dy**2)

        # 创建掩码：只处理在半径内且不在中心的点
        mask = (dist < radius) & (dist > 0)

        # 计算径向角度
        angle = np.arctan2(dy, dx)

        # 计算向量大小（从中心向外递减）
        vec_magnitude = magnitude * (1.0 - (dist / radius))

        # 计算向量分量
        vx = vec_magnitude * np.cos(angle)
        vy = vec_magnitude * np.sin(angle)

        # 应用到网格
        grid[mask, 0] += vx[mask]
        grid[mask, 1] += vy[mask]

        return grid

    def create_tangential_pattern(self, grid: np.ndarray, center: Tuple[float, float] = None,
                               radius: float = None, magnitude: float = 1.0) -> np.ndarray:
        """在网格上创建切线向量模式（旋转）"""
        if grid is None or not isinstance(grid, np.ndarray):
            raise TypeError("grid 必须是 numpy.ndarray 类型")

        h, w = grid.shape[:2]

        # 如果未指定中心，则使用网格中心
        if center is None:
            center = (w // 2, h // 2)

        # 如果未指定半径，则使用网格尺寸的1/4
        if radius is None:
            radius = min(w, h) // 4

        cx, cy = center

        # 创建坐标网格
        y_coords, x_coords = np.mgrid[0:h, 0:w]

        # 计算每个点到中心的距离和方向
        dx = x_coords - cx
        dy = y_coords - cy
        dist = np.sqrt(dx**2 + dy**2)

        # 创建掩码：只处理在半径内且不在中心的点
        mask = (dist < radius) & (dist > 0)

        # 计算切线角度（径向角度+90度）
        angle = np.arctan2(dy, dx) + np.pi/2

        # 计算向量大小（从中心向外递减）
        vec_magnitude = magnitude * (1.0 - (dist / radius))

        # 计算向量分量
        vx = vec_magnitude * np.cos(angle)
        vy = vec_magnitude * np.sin(angle)

        # 应用到网格
        grid[mask, 0] += vx[mask]
        grid[mask, 1] += vy[mask]

        return grid



"""
应用核心模块 - 提供应用程序的主要功能
整合各个管理器，提供统一的应用程序接口
"""
import os
import threading
import numpy as np
import time
from typing import Optional, Dict, Any, Tuple, Callable, Union
from .events import Event, EventType, event_bus, EventHandler, EventBus
from .state import StateManager, state_manager
from .config import ConfigManager, config_manager
from .container import container
from ..compute.vector_field import VectorFieldCalculator
from ..graphics.renderer import VectorFieldRenderer
from ..window.window import Window

class GridManager(EventHandler):
    """网格数据管理器"""
    def __init__(self, state_manager: StateManager, event_bus: "EventBus"):
        self._state_manager = state_manager
        self._event_bus = event_bus
        self._lock = threading.RLock()
        self._grid = None

        # 初始化网格状态
        self._state_manager.set("grid_width", 640)
        self._state_manager.set("grid_height", 480)
        self._state_manager.set("grid_updated", False)

        # 订阅事件
        self._event_bus.subscribe(EventType.CLEAR_GRID, self)
        self._event_bus.subscribe(EventType.TOGGLE_GRID, self)

    @property
    def grid(self) -> Optional[np.ndarray]:
        """获取网格数据的副本"""
        with self._lock:
            return self._grid.copy() if self._grid is not None else None

    def init_grid(self, width: int = 640, height: int = 480, default: Tuple[float, float] = (0.0, 0.0)) -> np.ndarray:
        """初始化网格"""
        with self._lock:
            self._grid = np.zeros((height, width, 2), dtype=np.float32)
            if default != (0.0, 0.0):
                self._grid[:, :, 0] = default[0]
                self._grid[:, :, 1] = default[1]

            # 更新状态
            self._state_manager.update({
                "grid_width": width,
                "grid_height": height,
                "grid_updated": True
            })

            # 发布事件
            self._event_bus.publish(Event(
                EventType.GRID_UPDATED,
                {"width": width, "height": height},
                "GridManager"
            ))

            return self._grid.copy()

    def update_grid(self, updates: Dict[Tuple[int, int], Tuple[float, float]]) -> None:
        """更新网格中的特定点"""
        with self._lock:
            if self._grid is None:
                return

            changed = False
            for (y, x), (vx, vy) in updates.items():
                if 0 <= y < self._grid.shape[0] and 0 <= x < self._grid.shape[1]:
                    self._grid[y, x] = (vx, vy)
                    changed = True

            if changed:
                # 更新状态
                self._state_manager.set("grid_updated", True)

                # 发布事件
                self._event_bus.publish(Event(
                    EventType.GRID_UPDATED,
                    {"updates": updates},
                    "GridManager"
                ))

    def clear_grid(self) -> None:
        """清空网格"""
        with self._lock:
            if self._grid is not None:
                self._grid.fill(0.0)

                # 更新状态
                self._state_manager.set("grid_updated", True, notify=False)

                # 发布事件
                self._event_bus.publish(Event(
                    EventType.GRID_CLEARED,
                    {},
                    "GridManager"
                ))

    def load_grid(self, file_path: str) -> bool:
        """从文件加载网格"""
        try:
            if not os.path.exists(file_path):
                print(f"[网格管理] 文件不存在: {file_path}")
                return False

            loaded_grid = np.load(file_path)

            with self._lock:
                if self._grid is not None and loaded_grid.shape != self._grid.shape:
                    print(f"[网格管理] 网格尺寸不匹配: {loaded_grid.shape} vs {self._grid.shape}")
                    return False

                self._grid = loaded_grid.copy()

                # 更新状态
                self._state_manager.update({
                    "grid_width": loaded_grid.shape[1],
                    "grid_height": loaded_grid.shape[0],
                    "grid_updated": True
                })

                # 发布事件
                self._event_bus.publish(Event(
                    EventType.GRID_LOADED,
                    {"file_path": file_path, "shape": loaded_grid.shape},
                    "GridManager"
                ))

                return True
        except Exception as e:
            print(f"[网格管理] 加载网格失败: {e}")
            return False

    def save_grid(self, file_path: str) -> bool:
        """保存网格到文件"""
        try:
            with self._lock:
                if self._grid is not None:
                    np.save(file_path, self._grid)

                    # 发布事件
                    self._event_bus.publish(Event(
                        EventType.GRID_SAVED,
                        {"file_path": file_path, "shape": self._grid.shape},
                        "GridManager"
                    ))

                    return True
            return False
        except Exception as e:
            print(f"[网格管理] 保存网格失败: {e}")
            return False

    def handle(self, event: Event) -> None:
        """处理事件"""
        if event.type == EventType.CLEAR_GRID:
            self.clear_grid()
        elif event.type == EventType.TOGGLE_GRID:
            show_grid = self._state_manager.get("show_grid", True)
            self._state_manager.set("show_grid", not show_grid)

class ViewManager(EventHandler):
    """视图管理器"""
    def __init__(self, state_manager: StateManager, event_bus: Event):
        self._state_manager = state_manager
        self._event_bus = event_bus

        # 初始化视图状态
        self._state_manager.update({
            "cam_x": 0.0,
            "cam_y": 0.0,
            "cam_zoom": 1.0,
            "view_changed": False,
        })

        # 订阅事件
        self._event_bus.subscribe(EventType.RESET_VIEW, self)

    def reset_view(self, width: int = 640, height: int = 480) -> None:
        """重置视图到网格中心"""
        cell_size = config_manager.get("cell_size", 1.0)
        cam_x = (width * cell_size) / 2.0
        cam_y = (height * cell_size) / 2.0
        cam_zoom = config_manager.get("cam_zoom", 1.0)

        # 更新状态
        self._state_manager.update({
            "cam_x": cam_x,
            "cam_y": cam_y,
            "cam_zoom": cam_zoom,
            "view_changed": True
        })

        # 发布事件
        self._event_bus.publish(Event(
            EventType.VIEW_RESET,
            {"cam_x": cam_x, "cam_y": cam_y, "cam_zoom": cam_zoom},
            "ViewManager"
        ))

    def handle(self, event: Event) -> None:
        """处理事件"""
        if event.type == EventType.RESET_VIEW:
            width = self._state_manager.get("grid_width", 640)
            height = self._state_manager.get("grid_height", 480)
            self.reset_view(width, height)

class AppCore:
    """应用核心类，整合各个管理器"""
    def __init__(self):
        self._state_manager = state_manager
        self._event_bus = event_bus
        self._config_manager = config_manager

        # 初始化各个管理器
        self._grid_manager = GridManager(self._state_manager, self._event_bus)
        self._view_manager = ViewManager(self._state_manager, self._event_bus)

        # 从容器获取服务
        self._vector_calculator = container.resolve(VectorFieldCalculator)
        self._renderer = container.resolve(VectorFieldRenderer)

        # 如果容器中没有这些服务，则创建它们
        if self._vector_calculator is None:
            self._vector_calculator = VectorFieldCalculator()
            container.register_singleton(VectorFieldCalculator, self._vector_calculator)

        if self._renderer is None:
            self._renderer = VectorFieldRenderer()
            container.register_singleton(VectorFieldRenderer, self._renderer)

        # 发布应用初始化事件
        self._event_bus.publish(Event(
            EventType.APP_INITIALIZED,
            {},
            "AppCore"
        ))

    @property
    def state_manager(self) -> StateManager:
        """获取状态管理器"""
        return self._state_manager

    @property
    def event_bus(self) -> Event:
        """获取事件总线"""
        return self._event_bus

    @property
    def config_manager(self) -> ConfigManager:
        """获取配置管理器"""
        return self._config_manager

    @property
    def grid_manager(self) -> GridManager:
        """获取网格管理器"""
        return self._grid_manager

    @property
    def view_manager(self) -> ViewManager:
        """获取视图管理器"""
        return self._view_manager

    @property
    def vector_calculator(self) -> VectorFieldCalculator:
        """获取向量计算器"""
        return self._vector_calculator

    @property
    def renderer(self) -> VectorFieldRenderer:
        """获取渲染器"""
        return self._renderer

    def shutdown(self) -> None:
        """关闭应用核心"""
        # 发布应用关闭事件
        self._event_bus.publish(Event(
            EventType.APP_SHUTDOWN,
            {},
            "AppCore"
        ))

        # 清理资源
        self._state_manager.clear_listeners()
        self._event_bus.clear()

        # 清理渲染器
        if self._renderer:
            self._renderer.cleanup()

# 注册服务到容器
container.register_singleton(StateManager, state_manager)
container.register_singleton(ConfigManager, config_manager)
container.register_singleton(Event, event_bus)
container.register_singleton(AppCore, AppCore)



"""
GPU向量场计算模块 - 提供基于GPU的向量场计算功能
使用OpenCL实现高性能计算
"""
import numpy as np
import pyopencl as cl
from typing import Tuple, Union, List, Optional, Any
from ..core.config import config_manager
from ..core.events import Event, EventType, event_bus

class GPUVectorFieldCalculator:
    """GPU向量场计算器，使用OpenCL实现"""
    def __init__(self):
        self._event_bus = event_bus
        self._config_manager = config_manager
        self._ctx = None
        self._queue = None
        self._programs = {}
        self._initialized = False

        # 初始化OpenCL
        self._init_opencl()

    def _init_opencl(self) -> None:
        """初始化OpenCL环境"""
        try:
            # 获取平台和设备
            platforms = cl.get_platforms()
            if not platforms:
                raise RuntimeError("未找到OpenCL平台")

            # 选择第一个平台
            platform = platforms[0]
            devices = platform.get_devices(cl.device_type.GPU)

            if not devices:
                # 如果没有GPU设备，尝试使用CPU设备
                devices = platform.get_devices(cl.device_type.CPU)
                if not devices:
                    raise RuntimeError("未找到可用的OpenCL设备")

            # 创建上下文和命令队列
            self._ctx = cl.Context(devices)
            self._queue = cl.CommandQueue(self._ctx)

            # 编译OpenCL程序
            self._compile_programs()

            self._initialized = True
            print("[GPU计算] OpenCL初始化成功")

            # 发布GPU计算器初始化事件
            self._event_bus.publish(Event(
                EventType.GPU_COMPUTE_STARTED,
                {"device": "gpu", "status": "initialized"},
                "GPUVectorFieldCalculator"
            ))
        except Exception as e:
            print(f"[GPU计算] OpenCL初始化失败: {e}")
            self._initialized = False

            # 发布GPU计算器初始化失败事件
            self._event_bus.publish(Event(
                EventType.GPU_COMPUTE_ERROR,
                {"device": "gpu", "status": "failed", "error": str(e)},
                "GPUVectorFieldCalculator"
            ))

    def _compile_programs(self) -> None:
        """编译OpenCL程序"""
        # 相邻向量求和程序
        sum_adjacent_kernel = """
        __kernel void sum_adjacent_vectors(
            __global const float2* grid,
            __global float2* result,
            const int width,
            const int height,
            const int x,
            const int y,
            const float self_weight,
            const float neighbor_weight,
            const int include_self,
            const int enable_average
        ) {
            const int idx = get_global_id(0);
            const int idy = get_global_id(1);

            if (idx >= width || idy >= height) {
                return;
            }

            float2 sum = (float2)(0.0f, 0.0f);
            int count = 0;

            // 检查自身
            if (include_self && x == idx && y == idy) {
                sum += grid[idy * width + idx] * self_weight;
                count++;
            }

            // 检查四个邻居
            int2 neighbors[4] = {(int2)(0, -1), (int2)(0, 1), (int2)(-1, 0), (int2)(1, 0)};

            for (int i = 0; i < 4; i++) {
                int nx = idx + neighbors[i].x;
                int ny = idy + neighbors[i].y;

                if (nx >= 0 && nx < width && ny >= 0 && ny < height) {
                    sum += grid[ny * width + nx] * neighbor_weight;
                    count++;
                }
            }

            // 如果启用平均值，则除以有效向量数量
            if (enable_average && count > 0) {
                sum /= (float)count;
            }

            result[idy * width + idx] = sum;
        }
        """

        # 更新网格程序
        update_grid_kernel = """
        __kernel void update_grid_with_adjacent_sum(
            __global float2* grid,
            const int width,
            const int height,
            const float self_weight,
            const float neighbor_weight,
            const int include_self,
            const int enable_average,
            const int enable_normalization
        ) {
            const int idx = get_global_id(0);
            const int idy = get_global_id(1);

            if (idx >= width || idy >= height) {
                return;
            }

            float2 sum = (float2)(0.0f, 0.0f);
            int neighbor_count = 4;

            // 检查四个邻居
            int2 neighbors[4] = {(int2)(0, -1), (int2)(0, 1), (int2)(-1, 0), (int2)(1, 0)};

            for (int i = 0; i < 4; i++) {
                int nx = idx + neighbors[i].x;
                int ny = idy + neighbors[i].y;

                if (nx >= 0 && nx < width && ny >= 0 && ny < height) {
                    sum += grid[ny * width + nx] * neighbor_weight;
                } else {
                    neighbor_count--;
                }
            }

            // 如果包含自身，添加自身贡献
            if (include_self) {
                sum += grid[idy * width + idx] * self_weight;
                neighbor_count++;
            }

            // 如果需要平均值，计算有效邻居数量并归一化
            if (enable_average) {
                if (neighbor_count > 0) {
                    sum /= (float)neighbor_count;
                }
            }
            // 如果需要权重归一化
            else if (enable_normalization) {
                float weight_sum = neighbor_count * neighbor_weight;
                if (include_self) {
                    weight_sum += self_weight;
                }

                // 确保权重总和大于0，避免除以0
                weight_sum = fmax(weight_sum, 0.1f);

                sum /= weight_sum;
            }

            grid[idy * width + idx] = sum;
        }
        """

        # 创建径向模式程序
        radial_pattern_kernel = """
        __kernel void create_radial_pattern(
            __global float2* grid,
            const int width,
            const int height,
            const float center_x,
            const float center_y,
            const float radius,
            const float magnitude
        ) {
            const int idx = get_global_id(0);
            const int idy = get_global_id(1);

            if (idx >= width || idy >= height) {
                return;
            }

            // 计算到中心的距离
            float dx = (float)idx - center_x;
            float dy = (float)idy - center_y;
            float dist = sqrt(dx * dx + dy * dy);

            // 只处理在半径内且不在中心的点
            if (dist >= radius || dist <= 0.0f) {
                return;
            }

            // 计算径向角度
            float angle = atan2(dy, dx);

            // 计算向量大小（从中心向外递减）
            float vec_magnitude = magnitude * (1.0f - (dist / radius));

            // 计算向量分量
            float vx = vec_magnitude * cos(angle);
            float vy = vec_magnitude * sin(angle);

            // 应用到网格
            grid[idy * width + idx] += (float2)(vx, vy);
        }
        """

        # 创建切线模式程序
        tangential_pattern_kernel = """
        __kernel void create_tangential_pattern(
            __global float2* grid,
            const int width,
            const int height,
            const float center_x,
            const float center_y,
            const float radius,
            const float magnitude
        ) {
            const int idx = get_global_id(0);
            const int idy = get_global_id(1);

            if (idx >= width || idy >= height) {
                return;
            }

            // 计算到中心的距离
            float dx = (float)idx - center_x;
            float dy = (float)idy - center_y;
            float dist = sqrt(dx * dx + dy * dy);

            // 只处理在半径内且不在中心的点
            if (dist >= radius || dist <= 0.0f) {
                return;
            }

            // 计算切线角度（径向角度+90度）
            float angle = atan2(dy, dx) + M_PI_2;

            // 计算向量大小（从中心向外递减）
            float vec_magnitude = magnitude * (1.0f - (dist / radius));

            // 计算向量分量
            float vx = vec_magnitude * cos(angle);
            float vy = vec_magnitude * sin(angle);

            // 应用到网格
            grid[idy * width + idx] += (float2)(vx, vy);
        }
        """

        # 编译程序
        try:
            self._programs['sum_adjacent_vectors'] = cl.Program(self._ctx, sum_adjacent_kernel).build()
            self._programs['update_grid_with_adjacent_sum'] = cl.Program(self._ctx, update_grid_kernel).build()
            self._programs['create_radial_pattern'] = cl.Program(self._ctx, radial_pattern_kernel).build()
            self._programs['create_tangential_pattern'] = cl.Program(self._ctx, tangential_pattern_kernel).build()
        except Exception as e:
            print(f"[GPU计算] OpenCL程序编译失败: {e}")
            raise

    def sum_adjacent_vectors(self, grid: np.ndarray, x: int, y: int, include_self: bool = None,
                           self_weight: float = 1.0, neighbor_weight: float = 0.1, 
                           enable_average: bool = False) -> Tuple[float, float]:
        """使用GPU计算相邻向量之和"""
        if not self._initialized:
            raise RuntimeError("GPU计算器未初始化")

        if grid is None:
            return (0.0, 0.0)

        if not isinstance(grid, np.ndarray):
            raise TypeError("grid 必须是 numpy.ndarray 类型")

        h, w = grid.shape[:2]

        # 创建输出缓冲区
        result = np.zeros((h, w, 2), dtype=np.float32)

        # 创建OpenCL缓冲区
        grid_buf = cl.Buffer(self._ctx, cl.mem_flags.READ_ONLY | cl.mem_flags.COPY_HOST_PTR, hostbuf=grid)
        result_buf = cl.Buffer(self._ctx, cl.mem_flags.WRITE_ONLY, result.nbytes)

        # 设置参数
        include_self_int = 1 if include_self else 0
        enable_average_int = 1 if enable_average else 0

        # 执行内核
        self._programs['sum_adjacent_vectors'].sum_adjacent_vectors(
            self._queue, (w, h), None,
            grid_buf, result_buf,
            np.int32(w), np.int32(h), np.int32(x), np.int32(y),
            np.float32(self_weight), np.float32(neighbor_weight),
            np.int32(include_self_int), np.int32(enable_average_int)
        )

        # 读取结果
        cl.enqueue_copy(self._queue, result, result_buf)

        # 返回指定位置的向量
        return (result[y, x, 0], result[y, x, 1])

    def update_grid_with_adjacent_sum(self, grid: np.ndarray, include_self: bool = None) -> np.ndarray:
        """使用GPU更新网格"""
        if not self._initialized:
            raise RuntimeError("GPU计算器未初始化")

        if grid is None or not isinstance(grid, np.ndarray):
            return grid

        h, w = grid.shape[:2]

        # 获取配置参数
        neighbor_weight = self._config_manager.get("vector_neighbor_weight", 0.1)
        self_weight = self._config_manager.get("vector_self_weight", 1.0)
        enable_average = self._config_manager.get("enable_vector_average", False)
        enable_normalization = self._config_manager.get("enable_vector_normalization", True)

        # 如果未指定include_self，则使用配置管理器中的默认值
        if include_self is None:
            include_self = self._config_manager.get("include_self", False)

        # 创建OpenCL缓冲区
        grid_buf = cl.Buffer(self._ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf=grid)

        # 设置参数
        include_self_int = 1 if include_self else 0
        enable_average_int = 1 if enable_average else 0
        enable_normalization_int = 1 if enable_normalization else 0

        # 执行内核
        self._programs['update_grid_with_adjacent_sum'].update_grid_with_adjacent_sum(
            self._queue, (w, h), None,
            grid_buf, np.int32(w), np.int32(h),
            np.float32(self_weight), np.float32(neighbor_weight),
            np.int32(include_self_int), np.int32(enable_average_int),
            np.int32(enable_normalization_int)
        )

        # 读取结果
        cl.enqueue_copy(self._queue, grid, grid_buf)

        return grid

    def create_vector_grid(self, width: int = 640, height: int = 480, default: Tuple[float, float] = (0, 0)) -> np.ndarray:
        """创建一个 height x width 的二维向量网格"""
        # GPU版本与CPU版本相同，因为初始化网格不需要GPU计算
        grid = np.zeros((height, width, 2), dtype=np.float32)
        if default != (0, 0):
            grid[:, :, 0] = default[0]
            grid[:, :, 1] = default[1]
        return grid

    def create_radial_pattern(self, grid: np.ndarray, center: Tuple[float, float] = None,
                            radius: float = None, magnitude: float = 1.0) -> np.ndarray:
        """使用GPU在网格上创建径向向量模式"""
        if not self._initialized:
            raise RuntimeError("GPU计算器未初始化")

        if grid is None or not isinstance(grid, np.ndarray):
            raise TypeError("grid 必须是 numpy.ndarray 类型")

        h, w = grid.shape[:2]

        # 如果未指定中心，则使用网格中心
        if center is None:
            center = (w // 2, h // 2)

        # 如果未指定半径，则使用网格尺寸的1/4
        if radius is None:
            radius = min(w, h) // 4

        cx, cy = center

        # 创建OpenCL缓冲区
        grid_buf = cl.Buffer(self._ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf=grid)

        # 执行内核
        self._programs['create_radial_pattern'].create_radial_pattern(
            self._queue, (w, h), None,
            grid_buf, np.int32(w), np.int32(h),
            np.float32(cx), np.float32(cy),
            np.float32(radius), np.float32(magnitude)
        )

        # 读取结果
        cl.enqueue_copy(self._queue, grid, grid_buf)

        return grid

    def create_tangential_pattern(self, grid: np.ndarray, center: Tuple[float, float] = None,
                               radius: float = None, magnitude: float = 1.0) -> np.ndarray:
        """使用GPU在网格上创建切线向量模式（旋转）"""
        if not self._initialized:
            raise RuntimeError("GPU计算器未初始化")

        if grid is None or not isinstance(grid, np.ndarray):
            raise TypeError("grid 必须是 numpy.ndarray 类型")

        h, w = grid.shape[:2]

        # 如果未指定中心，则使用网格中心
        if center is None:
            center = (w // 2, h // 2)

        # 如果未指定半径，则使用网格尺寸的1/4
        if radius is None:
            radius = min(w, h) // 4

        cx, cy = center

        # 创建OpenCL缓冲区
        grid_buf = cl.Buffer(self._ctx, cl.mem_flags.READ_WRITE | cl.mem_flags.COPY_HOST_PTR, hostbuf=grid)

        # 执行内核
        self._programs['create_tangential_pattern'].create_tangential_pattern(
            self._queue, (w, h), None,
            grid_buf, np.int32(w), np.int32(h),
            np.float32(cx), np.float32(cy),
            np.float32(radius), np.float32(magnitude)
        )

        # 读取结果
        cl.enqueue_copy(self._queue, grid, grid_buf)

        return grid

    def cleanup(self) -> None:
        """清理资源"""
        if self._ctx:
            del self._ctx
        if self._queue:
            del self._queue
        self._programs.clear()
        self._initialized = False



"""
鼠标映射 - 定义鼠标按钮的映射
"""
import glfw


class MouseMap:
    """鼠标映射类，提供鼠标按钮的常量定义"""

    # 鼠标按钮
    LEFT = glfw.MOUSE_BUTTON_LEFT
    RIGHT = glfw.MOUSE_BUTTON_RIGHT
    MIDDLE = glfw.MOUSE_BUTTON_MIDDLE
    _4 = glfw.MOUSE_BUTTON_4
    _5 = glfw.MOUSE_BUTTON_5
    _6 = glfw.MOUSE_BUTTON_6
    _7 = glfw.MOUSE_BUTTON_7
    _8 = glfw.MOUSE_BUTTON_8

    # 动作
    PRESS = glfw.PRESS
    RELEASE = glfw.RELEASE
    REPEAT = glfw.REPEAT

    @staticmethod
    def get_button_name(button: int) -> str:
        """获取鼠标按钮名称

        Args:
            button: GLFW鼠标按钮

        Returns:
            str: 按钮名称
        """
        button_names = {
            MouseMap.LEFT: "Left Button",
            MouseMap.RIGHT: "Right Button",
            MouseMap.MIDDLE: "Middle Button",
            MouseMap._4: "Button 4",
            MouseMap._5: "Button 5",
            MouseMap._6: "Button 6",
            MouseMap._7: "Button 7",
            MouseMap._8: "Button 8"
        }
        return button_names.get(button, "Unknown Button")

    @staticmethod
    def get_action_name(action: int) -> str:
        """获取动作名称

        Args:
            action: GLFW动作

        Returns:
            str: 动作名称
        """
        action_names = {
            MouseMap.PRESS: "Press",
            MouseMap.RELEASE: "Release",
            MouseMap.REPEAT: "Repeat"
        }
        return action_names.get(action, "Unknown Action")



"""
配置管理模块 - 提供统一的配置管理功能
支持从文件加载配置和热更新
"""
import json
import os
import sys
import threading
from typing import Any, Dict, Optional, Union, List
from dataclasses import dataclass, asdict, field
from .state import StateManager, state_manager
from .events import Event, EventType, event_bus

@dataclass
class ConfigOption:
    """配置选项"""
    key: str
    value: Any
    default: Any
    description: str = ""
    type: str = "string"  # string, number, boolean, array, object
    options: List[Any] = field(default_factory=list)
    min_value: Optional[Union[int, float]] = None
    max_value: Optional[Union[int, float]] = None

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return asdict(self)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'ConfigOption':
        """从字典创建配置选项"""
        return cls(**data)

class ConfigManager:
    """配置管理器"""
    def __init__(self, config_file: Optional[str] = None):
        self._config_file = config_file
        self._options: Dict[str, ConfigOption] = {}
        self._lock = threading.RLock()
        self._state_manager = state_manager
        self._event_bus = event_bus

        # 初始化默认配置
        self._init_default_config()

        # 如果指定了配置文件，加载配置
        if self._config_file and os.path.exists(self._config_file):
            self.load_from_file(self._config_file)

    def _init_default_config(self) -> None:
        """初始化默认配置"""
        # 网格配置
        self.register_option("grid_width", 640, "网格宽度", type="number")
        self.register_option("grid_height", 480, "网格高度", type="number")
        self.register_option("cell_size", 1.0, "单元格大小", type="number")

        # 向量场配置
        self.register_option("vector_color", [0.2, 0.6, 1.0], "向量颜色", type="array")
        self.register_option("vector_scale", 1.0, "向量缩放", type="number", min_value=0.1, max_value=10.0)
        self.register_option("vector_self_weight", 0.2, "向量自身权重", type="number", min_value=0.0, max_value=10.0)
        self.register_option("vector_neighbor_weight", 0.2, "向量邻居权重", type="number", min_value=0.0, max_value=10.0)
        self.register_option("include_self", True, "是否包含自身向量", type="boolean")
        self.register_option("enable_vector_average", False, "是否启用向量平均值", type="boolean")
        self.register_option("enable_vector_normalization", False, "是否启用向量归一化", type="boolean")

        # 视图配置
        self.register_option("cam_x", 0.0, "相机X坐标", type="number")
        self.register_option("cam_y", 0.0, "相机Y坐标", type="number")
        self.register_option("cam_zoom", 1.0, "相机缩放", type="number", min_value=0.1, max_value=10.0)
        self.register_option("show_grid", True, "是否显示网格", type="boolean")
        self.register_option("grid_color", [0.3, 0.3, 0.3], "网格颜色", type="array")

        # 渲染配置
        self.register_option("background_color", [0.1, 0.1, 0.1], "背景颜色", type="array")
        self.register_option("antialiasing", True, "是否启用抗锯齿", type="boolean")
        self.register_option("line_width", 1.0, "线条宽度", type="number", min_value=0.5, max_value=5.0)

        # 事件系统配置（移除未使用的项，保持事件总线行为默认）

        # 计算配置
        self.register_option("compute_device", "cpu", "计算设备", options=["cpu", "gpu"])
        self.register_option("compute_iterations", 1, "计算迭代次数", type="number", min_value=1, max_value=100)

        # UI 配置（简化：UI 主题/缩放由前端插件自行管理）

        # 渲染配置：是否渲染向量线条（新增）
        self.register_option("render_vector_lines", True, "是否渲染向量线条", type="boolean")

    def register_option(self, key: str, default: Any, description: str = "", 
                       type: str = "string", options: List[Any] = None,
                       min_value: Optional[Union[int, float]] = None,
                       max_value: Optional[Union[int, float]] = None) -> None:
        """注册配置选项"""
        with self._lock:
            if key in self._options:
                # 如果选项已存在，更新描述和其他元数据
                option = self._options[key]
                option.description = description
                option.type = type
                if options is not None:
                    option.options = options
                option.min_value = min_value
                option.max_value = max_value
            else:
                # 创建新选项
                option = ConfigOption(
                    key=key,
                    value=default,
                    default=default,
                    description=description,
                    type=type,
                    options=options or [],
                    min_value=min_value,
                    max_value=max_value
                )
                self._options[key] = option

            # 如果状态管理器中没有该配置，则设置默认值
            if not self._state_manager.contains(key):
                self._state_manager.set(key, default, notify=False)

    def get(self, key: str, default: Any = None) -> Any:
        """获取配置值"""
        with self._lock:
            if key in self._options:
                return self._state_manager.get(key, self._options[key].value)
            return self._state_manager.get(key, default)

    def set(self, key: str, value: Any) -> bool:
        """设置配置值"""
        with self._lock:
            if key not in self._options:
                print(f"[配置管理] 未知配置选项: {key}")
                return False

            option = self._options[key]

            # 类型检查
            if not self._validate_value(value, option):
                print(f"[配置管理] 配置值类型或范围不匹配: {key}")
                return False

            # 设置值
            old_value = self._state_manager.get(key, option.value)
            self._state_manager.set(key, value)

            # 如果值发生了变化，发布配置变更事件
            if old_value != value:
                self._event_bus.publish(Event(
                    EventType.CONFIG_CHANGED,
                    {"key": key, "old_value": old_value, "new_value": value},
                    "ConfigManager"
                ))

            # 如果配置文件已设置，保存配置
            if self._config_file:
                self.save_to_file(self._config_file)

            return True

    def _validate_value(self, value: Any, option: ConfigOption) -> bool:
        """验证配置值是否有效"""
        # 类型验证
        if option.type == "boolean":
            return isinstance(value, bool)
        elif option.type == "number":
            if not isinstance(value, (int, float)):
                return False
            # 范围验证
            if option.min_value is not None and value < option.min_value:
                return False
            if option.max_value is not None and value > option.max_value:
                return False
            return True
        elif option.type == "array":
            return isinstance(value, list)
        elif option.type == "object":
            return isinstance(value, dict)
        else:  # string
            return isinstance(value, str)

    def get_all(self) -> Dict[str, Any]:
        """获取所有配置"""
        with self._lock:
            config = {}
            for key, option in self._options.items():
                config[key] = self._state_manager.get(key, option.value)
            return config

    def _nest_dict_from_flat(self, flat: Dict[str, Any]) -> Dict[str, Any]:
        """将扁平键（使用下划线分隔）转换为嵌套字典

        例如: {'vector_color': [..], 'grid_width': 640} -> {'vector': {'color': [...]}, 'grid': {'width': 640}}
        """
        nested: Dict[str, Any] = {}
        for flat_key, value in flat.items():
            parts = flat_key.split('_') if flat_key else [flat_key]
            cur = nested
            for i, part in enumerate(parts):
                if i == len(parts) - 1:
                    cur[part] = value
                else:
                    if part not in cur or not isinstance(cur[part], dict):
                        cur[part] = {}
                    cur = cur[part]
        return nested

    def _flatten_dict(self, nested: Dict[str, Any], parent_key: str = '') -> Dict[str, Any]:
        """将嵌套字典转换为扁平键（使用下划线连接）"""
        items: Dict[str, Any] = {}
        for k, v in nested.items():
            new_key = f"{parent_key}_{k}" if parent_key else k
            if isinstance(v, dict):
                items.update(self._flatten_dict(v, new_key))
            else:
                items[new_key] = v
        return items

    def reset_to_default(self, key: Optional[str] = None) -> None:
        """重置配置为默认值"""
        with self._lock:
            if key is None:
                # 重置所有配置
                for option in self._options.values():
                    self._state_manager.set(option.key, option.default)

                # 发布配置重置事件
                self._event_bus.publish(Event(
                    EventType.CONFIG_CHANGED,
                    {"action": "reset_all"},
                    "ConfigManager"
                ))
            else:
                # 重置单个配置
                if key in self._options:
                    option = self._options[key]
                    self._state_manager.set(key, option.default)

                    # 发布配置重置事件
                    self._event_bus.publish(Event(
                        EventType.CONFIG_CHANGED,
                        {"key": key, "action": "reset"},
                        "ConfigManager"
                    ))

    def load_from_file(self, file_path: str) -> bool:
        """从文件加载配置"""
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                config_data = json.load(f)

            # 支持嵌套或扁平的配置文件格式
            flat_config: Dict[str, Any] = {}
            if isinstance(config_data, dict):
                # 将嵌套的 dict 扁平化为使用下划线的键名
                flat_config = self._flatten_dict(config_data)

            with self._lock:
                for key, value in flat_config.items():
                    if key in self._options:
                        self.set(key, value)
                    else:
                        # 兼容：也尝试直接使用原始键（如果文件已经是扁平的）
                        if key in self._options:
                            self.set(key, value)

            return True
        except Exception as e:
            print(f"[配置管理] 加载配置文件失败: {e}")
            return False

    def save_to_file(self, file_path: Optional[str] = None) -> bool:
        """保存配置到文件"""
        try:
            config_file = file_path or self._config_file
            if not config_file:
                return False

            # 获取扁平配置并转换为嵌套结构写入文件
            flat_config = self.get_all()
            nested = self._nest_dict_from_flat(flat_config)

            # 确保目录存在（仅当路径包含目录时创建）
            dir_name = os.path.dirname(config_file)
            if dir_name:
                os.makedirs(dir_name, exist_ok=True)

            with open(config_file, 'w', encoding='utf-8') as f:
                json.dump(nested, f, indent=4)

            return True
        except Exception as e:
            print(f"[配置管理] 保存配置文件失败: {e}")
            return False

    def get_option_info(self, key: str) -> Optional[Dict[str, Any]]:
        """获取配置选项信息"""
        with self._lock:
            if key in self._options:
                return self._options[key].to_dict()
            return None

    def get_all_option_info(self) -> Dict[str, Dict[str, Any]]:
        """获取所有配置选项信息"""
        with self._lock:
            return {key: option.to_dict() for key, option in self._options.items()}

    def __getitem__(self, key: str) -> Any:
        """通过索引获取配置"""
        return self.get(key)

    def __setitem__(self, key: str, value: Any) -> None:
        """通过索引设置配置"""
        self.set(key, value)

    def __contains__(self, key: str) -> bool:
        """检查配置是否存在"""
        return key in self._options

# 全局配置管理器实例
# 配置路径查找策略（打包后可动态覆盖）：
# 1. 环境变量 LIZI_CONFIG 或 LIZIENGINE_CONFIG
# 2. 当前工作目录下的 config.json
# 3. 可执行文件所在目录下的 config.json（对 PyInstaller/frozen 生效）
env_path = os.environ.get("LIZI_CONFIG") or os.environ.get("LIZIENGINE_CONFIG")
if env_path:
    config_path = env_path
else:
    cwd_path = os.path.join(os.getcwd(), "config.json")
    if os.path.exists(cwd_path):
        config_path = cwd_path
    else:
        if getattr(sys, 'frozen', False):
            base_dir = os.path.dirname(sys.executable)
        else:
            base_dir = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
        config_path = os.path.join(base_dir, "config.json")

config_manager = ConfigManager(config_path)



"""
依赖注入容器 - 提供依赖注入功能，避免单例模式，降低模块间耦合
"""
from typing import Dict, Type, Any, Callable, Optional, TypeVar, Generic, get_type_hints
from inspect import isclass, isfunction, ismethod
import threading

T = TypeVar('T')

class ServiceDescriptor:
    """服务描述符，包含服务的生命周期和工厂方法"""
    def __init__(self, factory: Callable, singleton: bool = True):
        self.factory = factory
        self.singleton = singleton
        self.instance = None if singleton else None
        self._lock = threading.Lock()

    def get_instance(self, container: 'Container') -> Any:
        """获取服务实例"""
        if not self.singleton:
            return self._create_instance(container)

        with self._lock:
            if self.instance is None:
                self.instance = self._create_instance(container)
            return self.instance

    def _create_instance(self, container: 'Container') -> Any:
        """创建服务实例"""
        if isclass(self.factory):
            # 如果是类，解析构造函数参数并自动注入
            return self._create_with_injection(container, self.factory)
        else:
            # 如果是工厂函数，调用它并自动注入参数
            return self._create_with_injection(container, self.factory)

    def _create_with_injection(self, container: 'Container', factory: Callable) -> Any:
        """通过依赖注入创建实例"""
        # 获取函数或类的类型提示
        hints = get_type_hints(factory)

        # 获取参数列表
        if isclass(factory):
            # 类的构造函数
            import inspect
            params = inspect.signature(factory.__init__).parameters
        else:
            # 函数
            import inspect
            params = inspect.signature(factory).parameters

        # 准备参数字典
        kwargs = {}

        for name, param in params.items():
            if name == 'self':
                continue

            # 检查是否有类型提示
            param_type = hints.get(name)
            if param_type:
                # 尝试从容器中解析依赖
                dependency = container.resolve(param_type)
                if dependency is not None:
                    kwargs[name] = dependency
                elif param.default is inspect.Parameter.empty:
                    # 如果没有默认值且无法解析依赖，抛出异常
                    raise ValueError(f"无法解析依赖: {name} ({param_type})")
            elif param.default is not inspect.Parameter.empty:
                # 如果没有类型提示但有默认值，使用默认值
                kwargs[name] = param.default

        # 创建实例
        if isclass(factory):
            return factory(**kwargs)
        else:
            return factory(**kwargs)

class Container:
    """依赖注入容器"""
    def __init__(self):
        self._services: Dict[Type, ServiceDescriptor] = {}
        self._instances: Dict[Type, Any] = {}
        self._lock = threading.Lock()

    def register(self, service_type: Type[T], factory: Callable, singleton: bool = True) -> None:
        """注册服务"""
        with self._lock:
            self._services[service_type] = ServiceDescriptor(factory, singleton)

    def register_singleton(self, service_type: Type[T], instance: T) -> None:
        """注册单例实例"""
        with self._lock:
            self._services[service_type] = ServiceDescriptor(lambda: instance, singleton=True)
            self._instances[service_type] = instance

    def register_transient(self, service_type: Type[T], factory: Callable) -> None:
        """注册瞬态服务（每次获取都创建新实例）"""
        with self._lock:
            self._services[service_type] = ServiceDescriptor(factory, singleton=False)

    def resolve(self, service_type: Type[T]) -> Optional[T]:
        """解析服务"""
        with self._lock:
            if service_type not in self._services:
                return None

            descriptor = self._services[service_type]
            return descriptor.get_instance(self)

    def is_registered(self, service_type: Type[T]) -> bool:
        """检查服务是否已注册"""
        with self._lock:
            return service_type in self._services

    def remove(self, service_type: Type[T]) -> None:
        """移除服务"""
        with self._lock:
            if service_type in self._services:
                descriptor = self._services.pop(service_type)
                if descriptor.singleton and descriptor.instance is not None:
                    # 如果是单例且已创建实例，尝试调用其cleanup方法
                    instance = descriptor.instance
                    if hasattr(instance, 'cleanup'):
                        try:
                            instance.cleanup()
                        except Exception:
                            pass
                if service_type in self._instances:
                    self._instances.pop(service_type)

    def clear(self) -> None:
        """清除所有服务"""
        with self._lock:
            # 清理所有单例实例
            for service_type, descriptor in self._services.items():
                if descriptor.singleton and descriptor.instance is not None:
                    # 如果是单例且已创建实例，尝试调用其cleanup方法
                    instance = descriptor.instance
                    if hasattr(instance, 'cleanup'):
                        try:
                            instance.cleanup()
                        except Exception:
                            pass
                    self._instances.pop(service_type, None)

            # 清除所有服务描述符
            self._services.clear()

# 全局容器实例
container = Container()
